@@ -24,15 +24,12 @@
 #
 
 package LEGO::NXT;
-use strict;
+use strict qw(vars);
 use warnings;
 
-use Net::Bluetooth;
-#Not ready for USB yet...
-#use Device::USB;
 use LEGO::NXT::Constants;
 
-our $VERSION = '1.42';
+our $VERSION = '2.00';
 our @ISA;
 
 =head1 NAME
@@ -104,6 +101,7 @@ to you.
 =cut
 
 my %error_codes = (
+  #Direct Command errors
   0x20 => "Pending communication transaction in progress",
   0x40 => "Specified mailbox queue is empty",
   0xBD => "Request failed (i.e. specified file not found)",
@@ -120,14 +118,36 @@ my %error_codes = (
   0xEF => "Attempted to access invalid field of a structure",
   0xF0 => "Bad input or output specified",
   0xFB => "Insufficient memory available",
-  0xFF => "Bad arguments"
+  0xFF => "Bad arguments",
+
+  #System errors
+  0x81 => "No more handles", 
+  0x82 => "No space",
+  0x83 => "No more files",
+  0x84 => "End of file expected",
+  0x85 => "End of file",
+  0x86 => "Not a linear file",
+  0x87 => "File not found",
+  0x88 => "Handle all ready closed",
+  0x89 => "No linear space",
+  0x8A => "Undefined error",
+  0x8B => "File is busy",
+  0x8C => "No write buffers",
+  0x8D => "Append not possible",
+  0x8E => "File is full",
+  0x8F => "File exists", 
+  0x90 => "Module not found",
+  0x91 => "Out of boundary",
+  0x92 => "Illegal file name",
+  0x93 => "Illegal handle"
 );
 
 =head1 METHODS
 
 =head2 new
 
-  $nxt = LEGO::NXT->new( 'xx:xx:xx:xx:xx:xx', 1 );
+  $nxt = LEGO::NXT->new( new LEGO::NXT:BlueComm('xx:xx:xx:xx',0) );
+  $nxt = LEGO::NXT->new( new LEGO::NXT:USBComm() );
 
 Creates a new NXT object, however a connection is not established until
 the first direct command is issued. Argument 1 should be the bluetooth
@@ -138,11 +158,10 @@ the channel you wish to connect on -- 1 or 2 seems to work.
 
 sub new
 {
-  my ($pkgnm,$btaddr,$channel) = @_;
+  my ($pkgnm,$comm) = @_;
+
   my $this = {
-    'btaddr'  => $btaddr,
-    'channel' => $channel,
-    'fh'      => undef,
+    'comm'    => $comm,
     'error'   => undef,
     'errstr'  => undef,
     'status'  => undef,
@@ -412,7 +431,6 @@ sub start_program
   my ($this,$needsret,$file) = @_;
   my $strlen = 1+length($file);
   my $ret    = $this->_do_cmd(
-    pack("v",3+$strlen).
     pack("CCZ[$strlen]",$needsret,$NXT_START_PROGRAM,$file),
     $needsret
   );
@@ -435,7 +453,6 @@ sub stop_program
   my ($this,$needsret) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",2).
     pack("CC",$needsret,$NXT_STOP_PROGRAM),
     $needsret
   );
@@ -457,8 +474,7 @@ sub play_tone
   my ($this,$needsret,$pitch,$duration) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v", 6).
-    pack("CCvv",0x80,$NXT_PLAY_TONE,$pitch,$duration),
+    pack("CCvv",$needsret,$NXT_PLAY_TONE,$pitch,$duration),
     $needsret
   );
 
@@ -480,7 +496,6 @@ sub play_sound_file
   my ($this,$needsret,$repeat,$file) = @_;
   my $strlen = 1+length($file);
   my $ret    = $this->_do_cmd( 
-    pack("v",3+$strlen).
     pack("CCCZ[$strlen]",$needsret,$NXT_PLAY_SOUND_FILE,$repeat,$file), 
     $needsret
   );
@@ -509,7 +524,6 @@ sub set_output_state
 {
   my ($this,$needsret,$port,$power,$mode,$regulation,$turnratio,$runstate,$tacholimit) = @_;
   my $ret = $this->_do_cmd(
-    pack("v",12).
     pack("CCCcCCcCV",$needsret,$NXT_SET_OUTPUT_STATE,$port,$power,$mode,$regulation,$turnratio,$runstate,$tacholimit),
     $needsret
   );
@@ -536,7 +550,6 @@ sub set_input_mode
   my ($this,$needsret,$port,$sensor_type,$sensor_mode) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",5).
     pack("CCCCC",$needsret,$NXT_SET_INPUT_MODE,$port,$sensor_type,$sensor_mode),
     $needsret
   );
@@ -560,7 +573,6 @@ sub get_output_state
 {
   my ($this,$needsret,$port) = @_;
   my $ret = $this->_do_cmd(
-    pack("v",3).
     pack("CCC",$needsret,$NXT_GET_OUTPUT_STATE,$port),
     $needsret
   );
@@ -584,7 +596,6 @@ sub get_input_values
 {
   my ($this,$needsret,$port) = @_;
   my $ret = $this->_do_cmd(
-    pack("v",3).
     pack("CCC",$needsret,$NXT_GET_INPUT_VALUES,$port),
     $needsret
   );
@@ -605,7 +616,6 @@ sub reset_input_scaled_value
 {
   my ($this,$needsret,$port) = @_;
   my $ret = $this->_do_cmd(
-    pack("v",3).
     pack("CCC",$needsret,$NXT_RESET_SCALED_INPUT_VALUE,$port),
     $needsret
   );
@@ -628,7 +638,6 @@ sub message_write
   my $mlen = 1+length($message);
 
   my $ret = $this->_do_cmd(
-    pack("v",4+$mlen).
     pack("CCCCZ[$mlen]",$needsret,$NXT_MESSAGE_WRITE,$mailbox,$mlen,$message),
     $needsret
   );
@@ -650,7 +659,6 @@ sub reset_motor_position
   my ($this,$needsret,$port,$relative) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",4).
     pack("CCCC",$needsret,$NXT_RESET_MOTOR_POSITION,$port,$relative),
     $needsret
   );
@@ -672,7 +680,6 @@ sub get_battery_level
   my ($this,$needsret) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",2).
     pack("CC",$needsret,$NXT_GET_BATTERY_LEVEL),
     $needsret
   );
@@ -694,7 +701,6 @@ sub set_stop_sound_playback
   my ($this,$needsret) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",2).
     pack("CC",$needsret,$NXT_STOP_SOUND_PLAYBACK),
     $needsret
   );
@@ -716,7 +722,6 @@ sub keep_alive
   my ($this,$needsret) = @_;
   
   my $ret = $this->_do_cmd(
-    pack("v",2).
     pack("CC",$needsret,$NXT_KEEP_ALIVE),
     $needsret	    
   );
@@ -739,7 +744,6 @@ sub ls_get_status
   my ($this,$needsret,$port) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",3).
     pack("CCC",$needsret,$NXT_LSGET_STATUS,$port),
     $needsret
   );
@@ -771,7 +775,6 @@ sub ls_write
   my ($this,$needsret,$port,$txlen,$rxlen,$txdata) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",5+$txlen).
     pack("CCCCC",$needsret,$NXT_LSWRITE,$port,$txlen,$rxlen).
     $txdata,
     $needsret
@@ -794,7 +797,6 @@ sub ls_read
   my ($this,$needsret,$port) = @_;
 
   my $ret = $this->_do_cmd(
-    pack("v",3).
     pack("CCC",$needsret,$NXT_LSREAD,$port),
     $needsret
   );
@@ -816,7 +818,7 @@ sub ls_request_response
 {
   my ($this,$port,$txlen,$rxlen,$data) = @_;
 
-  $this->ls_write($NXT_NORET,$port,$txlen,$rxlen,$data);
+  $this->ls_write($NXT_RET,$port,$txlen,$rxlen,$data);
 
   my $lsstat;
 
@@ -838,7 +840,6 @@ sub get_current_program_name
   my ($this,$needsret) = @_;
 
   my $ret = $this->_do_cmd(
-     pack("v",2).
      pack("CC",$needsret,$NXT_GET_CURRENT_PROGRAM_NAME),
      $needsret
   );
@@ -860,7 +861,6 @@ sub message_read
   my ($this,$needsret,$remotebox,$localbox,$remove) = @_;
   
   my $ret = $this->_do_cmd(
-    pack("v",5).
     pack("CCCCC",$needsret,$NXT_MESSAGE_READ,$remotebox,$localbox,$remove),
     $needsret
   );
@@ -869,6 +869,563 @@ sub message_read
   $this->_parse_message_read($ret);
 }
 
+=head1 NXT SYSTEM COMMANDS
+
+Caution. Use these only if you know what you're doing.
+
+If you know what you're doing, these methods can be very useful. If you don't know 
+what you're doing, you will probably end up with a dead robot. Beware. Seriously.
+
+NOTE: Every system command requires a return value, so there's no need to pass NXT_RET.
+
+=head2 sys_open_read
+
+  $ret = $nxt->sys_open_read($filename);
+
+Opens a system file for reading, returns a file descriptor.
+
+=cut
+
+sub sys_open_read
+{
+  my ($this,$file) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_OPEN_READ,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd,$file_size) = unpack( "vvCCV", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd,
+    'file_size' => $file_size
+  };
+}
+
+=head2 sys_open_write
+
+  $ret = $nxt->sys_open_write($filename,$size);
+
+Opens (creates?) a system file for writing, returns a file descriptor. You must specify 
+the size of the file you wish to write to.
+
+=cut
+
+sub sys_open_write
+{
+  my ($this,$file,$size) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]V",$NXT_SYSOP,$NXT_SYS_OPEN_WRITE,$file,$size),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd) = unpack( "vvCC", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd
+  };
+}
+
+=head2 sys_read
+
+  $ret = $nxt->sys_read($fd,$nbytes);
+
+Reads $nbytes from open file descriptor $fd. I believe $nbytes should be < 60 when using USB.
+
+=cut 
+
+sub sys_read
+{
+  my ($this,$fd,$nbytes) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CCCv",$NXT_SYSOP,$NXT_SYS_READ,$fd,$nbytes),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$nread,$data) = unpack( "vvCvC*", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'nread'     => $nread,
+    'data'      => $data
+  };
+} 
+
+=head2 sys_write
+
+  $ret = $nxt->sys_write($fd,$data);
+
+Writes $data to open file descriptor $fd.
+
+=cut
+
+sub sys_write
+{
+  my ($this,$fd,$data) = @_;
+
+  my $dlen   = 1+length($data);
+  my $ret    = $this->_do_cmd(
+    pack("CCCC[$dlen]",$NXT_SYSOP,$NXT_SYS_WRITE,$fd,$data),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$nfd,$nwritten) = unpack( "vvCCv", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'nwritten'  => $nwritten,
+    'fd'        => $nfd
+  };
+}
+
+=head2 sys_close
+
+  $ret = $nxt->sys_close($fd);
+
+Closes the file descriptor $fd.
+
+=cut
+
+sub sys_close
+{
+  my ($this,$fd) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CCC",$NXT_SYSOP,$NXT_SYS_CLOSE,$fd),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$nfd) = unpack( "vvCC", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $nfd
+  };
+}
+
+=head2 sys_delete
+
+  $ret = $nxt->sys_delete($filename);
+
+Deletes a file on the NXT with name $filename.
+
+=cut
+
+sub sys_delete
+{
+  my ($this,$file) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_DELETE,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$nfile) = unpack( "vvCZ[$strlen]", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'file'      => $nfile
+  };
+}
+
+
+=head2 sys_find_first
+
+  $ret = $nxt->sys_find_first($search);
+
+Finds a file on the system with filename $search. Searches may use wildcards: *.*, *.txt, etc. 
+This command will return statstr=>"File not found" on failure. The filehandle MUST be closed 
+when finished with the query.
+
+=cut
+
+sub sys_find_first
+{
+  my ($this,$file) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_FIND_FIRST,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd,$fname,$fsize) = unpack( "vvCCZ[20]V", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd,
+    'file'      => $fname,
+    'file_size' => $fsize
+  };
+}
+
+=head2 sys_find_next
+
+  $ret = $nxt->sys_find_next($fd);
+
+Finds the next matching file on the system from sys_find_first. 
+$fd is the descriptor returned from sys_find_first
+
+=cut
+
+sub sys_find_next
+{
+  my ($this,$fd) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CCC",$NXT_SYSOP,$NXT_SYS_FIND_NEXT,$fd),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$nfd,$fname,$fsize) = unpack( "vvCCZ[19]V", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $nfd,
+    'file'      => $fname,
+    'file_size' => $fsize
+  };
+}
+
+=head2 sys_get_firmware_version
+
+  $ret = $nxt->sys_get_firmware_version();
+
+Does what it says.
+
+=cut
+
+sub sys_get_firmware_version
+{
+  my ($this) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CC",$NXT_SYSOP,$NXT_SYS_GET_FIRMWARE_VERSION),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$pminor,$pmajor,$fminor,$fmajor) = unpack( "vvCCCC", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'protocol_version' => "$pmajor.$pminor",
+    'firmware_version' => "$fmajor.$fminor"
+  };
+}
+
+=head2 sys_open_write_linear
+
+  $ret = $nxt->sys_open_write_linear($filename,$size);
+
+Opens a system file for writing (raw data mode? update me... ), returns a file descriptor. You must specify
+the size of the file you wish to write to.
+
+=cut
+
+sub sys_open_write_linear
+{
+  my ($this,$file,$size) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]V",$NXT_SYSOP,$NXT_SYS_OPEN_WRITE_LINEAR,$file,$size),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd) = unpack( "vvCC", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd
+  };
+}
+
+=head2 sys_open_read_linear
+
+  $ret = $nxt->sys_open_read_linear($filename);
+
+Opens a system file for reading, returns the memory address of the file (NOT FD).
+
+=cut
+
+sub sys_open_read_linear
+{
+  my ($this,$file) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_OPEN_READ_LINEAR,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$ptr) = unpack( "vvCV", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'address'   => $ptr
+  };
+}
+
+
+=head2 sys_open_write_data
+
+  $ret = $nxt->sys_open_write_data($filename,$size);
+
+Opens a system file for writing (data mode? update me... ), returns a file descriptor. You must specify
+the size of the file you wish to write to.
+
+=cut
+
+sub sys_open_write_data
+{
+  my ($this,$file,$size) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]V",$NXT_SYSOP,$NXT_SYS_OPEN_WRITE_DATA,$file,$size),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd) = unpack( "vvCC", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd
+  };
+}
+
+=head2 sys_open_append_data
+
+  $ret = $nxt->sys_open_append_data($filename);
+
+Opens a system file for appendinf (data mode?), returns the fd and available size.
+
+=cut
+
+sub sys_open_append_data
+{
+  my ($this,$file) = @_;
+
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_OPEN_APPEND_DATA,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$fd,$size) = unpack( "vvCCV", $ret );
+
+  return
+  {
+    'status'    => $status,
+    'statstr'   => $status>0 ? $error_codes{$status} : 'ok',
+    'fd'        => $fd,
+    'size'      => $size
+  };
+}
+
+
+
+=head2 sys_boot
+
+  $ret = $nxt->sys_boot()
+
+Boot the NXT. DO NOT USE UNLESS YOU **REALLY** KNOW WHAT YOU'RE DOING!!! YOU CAN BREAK YOUR NXT!!! YOU HAVE BEEN WARNED.
+ 
+NOTE: USB Interface ONLY!
+
+=cut
+
+sub sys_boot
+{ 
+  my ($this) = @_;
+  
+  my $file   = "Let's dance: SAMBA"; 
+  my $strlen = 1+length($file);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_BOOT,$file),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$msg) = unpack( "vvCA[3]", $ret );
+
+  return
+  {
+    'status'      => $status,
+    'statstr'     => $status>0 ? $error_codes{$status} : 'ok',
+    'message'     => $msg
+  };
+  
+}
+
+
+=head2 sys_set_brick_name
+
+  $ret = $nxt->sys_set_brick_name($filename);
+
+Does what it says. I believe this defaults to NXT and is used in Bluetooth identification.
+
+=cut
+
+sub sys_set_brick_name
+{
+  my ($this,$name) = @_;
+
+  my $strlen = 1+length($name);
+  my $ret    = $this->_do_cmd(
+    pack("CCZ[$strlen]",$NXT_SYSOP,$NXT_SYS_SET_BRICK_NAME,$name),
+    $NXT_RET
+  );
+
+  $this->_parse_generic_ret($ret);
+}
+
+=head2 sys_get_device_info
+
+  $ret = $nxt->sys_get_device_info();
+
+Returns various informations about the NXT including Bluetooth info and available flash.
+
+=cut
+
+sub sys_get_device_info
+{
+  my ($this) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CC",$NXT_SYSOP,$NXT_SYS_GET_DEVICE_INFO),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$status,$name,$b1,$b2,$b3,$b4,$b5,$b6,$btstrength,$flash) = unpack( "vvCZ[15]"."CCCCCC"."VV", $ret );
+
+  return
+  {
+    'status'      => $status,
+    'statstr'     => $status>0 ? $error_codes{$status} : 'ok',
+    'name'        => $name,
+    'btaddr'      => join ':', map{ sprintf("%x",$_) } ($b1,$b2,$b3,$b4,$b5,$b6),
+    'btstrength'  => $btstrength,
+    'flash_free'  => $flash
+  };
+}
+
+=head2 sys_poll_command_length
+
+  $ret = $nxt->sys_poll_command_length($buffer_number);
+
+Polls for the number of bytes ready in the command buffer. 
+$buffer_number = 0x00 for poll buffer
+$buffer_number = 0xA1 for the high speed buffer
+* I have no idea what this does as of this writing.
+
+=cut
+
+sub sys_poll_command_length
+{
+  my ($this,$bn) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CCC",$NXT_SYSOP,$NXT_SYS_POLL_COMMAND_LENGTH,$bn),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$nbn,$status,$bytes) = unpack( "vvCCC", $ret );
+
+  return
+  {
+    'status'      => $status,
+    'statstr'     => $status>0 ? $error_codes{$status} : 'ok',
+    'buffer_number' => $nbn,
+    'bytes_ready'   => $bytes
+  };
+}
+
+=head2 sys_poll_command
+
+  $ret = $nxt->sys_poll_command($buffer_number,$command_length);
+
+Polls the command buffer.
+$buffer_number = 0x00 for poll buffer
+$buffer_number = 0xA1 for the high speed buffer
+* I have no idea what this does as of this writing.
+
+=cut
+
+sub sys_poll_command
+{
+  my ($this,$bn,$length) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CCCC",$NXT_SYSOP,$NXT_SYS_POLL_COMMAND_LENGTH,$bn,$length),
+    $NXT_RET
+  );
+
+  my ($len,$rval,$nbn,$status,$bytes,@command) = unpack( "vvCCCC*", $ret );
+
+  return
+  {
+    'status'        => $status,
+    'statstr'       => $status>0 ? $error_codes{$status} : 'ok',
+    'buffer_number' => $nbn,
+    'length'        => $bytes,
+    'command'       => join '', @command
+  };
+}
+
+=head2 sys_bluetooth_factory_reset
+
+  $ret = $nxt->sys_bluetooth_factory_reset($filename);
+
+Resets the on-board bluetooth chip to factory defaults.
+
+=cut
+
+sub sys_bluetooth_factory_reset
+{
+  my ($this) = @_;
+
+  my $ret    = $this->_do_cmd(
+    pack("CC",$NXT_SYSOP,$NXT_SYS_BLUETOOTH_FACTORY_RESET),
+    $NXT_RET
+  );
+
+  $this->_parse_generic_ret($ret);
+}
+
+
+
 =head1 PRIVATE METHODS
 
 =head2 _do_cmd
@@ -879,57 +1436,30 @@ sub _do_cmd
 {
   my ($this,$msg,$needsret) = @_;
 
-  $this->_bt_connect() unless defined $this->{fh};
+  my $comm = $this->{comm};
+  my $res;
 
-  my $fh = $this->{fh};
-  
-  syswrite( $fh, $msg, length $msg );
-  return if( $needsret == $NXT_NORET );
-  
-  #Begin reading response, if requested.
-  
-  my ($rin, $rout) = ('',''); 
-  my $rbuff;
-  my $total;
-
-  vec ($rin, fileno($fh), 1) = 1;
-
-  while( select($rout=$rin, undef, undef, 1) )
+  if ($comm->type eq 'usb')
   {
-    my $char = '';
-    my $nread=0;
-    eval
+    $res = $comm->do_cmd( $msg, $needsret );
+
+    #parsing functions expect prepended length bytes ala bluetooth
+    if (defined $res )
     {
-      local $SIG{ALRM} = sub { die "alarm\n" };
-      alarm 1;
-      $nread = sysread $fh, $char, 1;
-      alarm 0;
-    };
-    
-    $rbuff .= $char;
-  }
-  
-  return $rbuff;
-}
-
-=head2 _bt_connect
-
-=cut
-
-sub _bt_connect
-{
-  my ($this) = @_;
-  my $bt = Net::Bluetooth->newsocket("RFCOMM");
-  die "Socket could not be created!" unless(defined($bt));
-
-  if($bt->connect($this->{btaddr}, $this->{channel} ) != 0) {
-      die "connect error: $!";
+      $res = pack('v',length($res)).$res;
+    }
   }
 
-  $this->{fh} = $bt->perlfh();
-  $| = 1; #just in case our pipes are not already hot.
+  if ($comm->type eq 'blue')
+  {
+    #bluetooth comm requires prepended length bytes
+    $res = $comm->do_cmd( pack( "v", length($msg) ) . $msg, $needsret );
+  }
+
+  $res;
 }
 
+
 =head2 _parse_get_output_state
 
 =cut
@@ -1113,10 +1643,16 @@ sub _parse_generic_ret
   return
   {
     'status'            => $status,
-    'statstr'           => $status>0 ? $error_codes{$status} : 'ok'
+    'statstr'           => $status ? $error_codes{$status} : 'ok'
   };
 }
 
+sub _status_str
+{
+  my $status = shift;
+  $status>0 ? $error_codes{$status} : 'ok';  
+}
+
 =head2 import
 
 This is a custom import method for supporting
